@@ -36,6 +36,7 @@ def from_buffers(
     backend="cpu",
     byteorder="<",
     allow_noncanonical_form=False,
+    enable_virtualarray_caching=True,
     highlevel=True,
     behavior=None,
     attrs=None,
@@ -63,6 +64,14 @@ def from_buffers(
         allow_noncanonical_form (bool): If True, non-canonical forms will be
             simplified to produce arrays with canonical layouts; otherwise,
             an exception will be thrown for such forms.
+        enable_virtualarray_caching (bool or callable): If True (the default),
+            all VirtualNDArray buffers that get created will cache their
+            materialized buffers on themselves when they get materialized.
+            If a callable is given, it must accept two arguments, `form_key` and `attribute`,
+            and return a boolean indicating whether caching should be enabled for the
+            given buffer. The `form_key` and `attribute` are the same as those passed to
+            the `buffer_key` function. If False, all VirtualNDArrays will not cache
+            their materialized buffers on themselves.
         highlevel (bool): If True, return an #ak.Array; otherwise, return
             a low-level #ak.contents.Content subclass.
         behavior (None or dict): Custom #ak.behavior for the output array, if
@@ -118,6 +127,7 @@ def from_buffers(
         behavior,
         attrs,
         allow_noncanonical_form,
+        enable_virtualarray_caching,
     )
 
 
@@ -132,6 +142,7 @@ def _impl(
     behavior,
     attrs,
     simplify,
+    enable_virtualarray_caching,
 ):
     backend = regularize_backend(backend)
 
@@ -152,6 +163,17 @@ def _impl(
             "'form' argument must be a Form or its Python dict/JSON string representation"
         )
 
+    if isinstance(enable_virtualarray_caching, bool):
+
+        def enable_caching_function(form_key, attribute):
+            return enable_virtualarray_caching
+    elif callable(enable_virtualarray_caching):
+        enable_caching_function = enable_virtualarray_caching
+    else:
+        raise TypeError(
+            "'enable_virtualarray_caching' argument must be a boolean or a callable"
+        )
+
     getkey = regularize_buffer_key(buffer_key)
 
     out = _reconstitute(
@@ -162,8 +184,8 @@ def _impl(
         backend,
         byteorder,
         simplify,
-        field_path=(),
         shape_generator=lambda: (length,),
+        enable_virtualarray_caching=enable_caching_function,
     )
 
     return wrap_layout(out, highlevel=highlevel, attrs=attrs, behavior=behavior)
@@ -175,8 +197,9 @@ def _from_buffer(
     dtype: np.dtype,
     count: ShapeItem,
     byteorder: str,
-    field_path: tuple,
+    buffer_key: str,
     shape_generator: Callable | None = None,
+    enable_virtualarray_caching: bool = False,
 ) -> ArrayLike:
     if isinstance(buffer, VirtualNDArray):
         # This is the case for VirtualNDArrays
@@ -208,7 +231,14 @@ def _from_buffer(
         def generator():
             (length,) = cached_shape_generator()
             return _from_buffer(
-                nplike, buffer(), dtype, length, byteorder, field_path, None
+                nplike,
+                buffer(),
+                dtype,
+                length,
+                byteorder,
+                buffer_key,
+                None,
+                False,
             )
 
         # also store a ref to the original/raw buffer generator
@@ -221,7 +251,9 @@ def _from_buffer(
             dtype=dtype,
             generator=generator,
             shape_generator=cached_shape_generator,
+            buffer_key=buffer_key,
             __wrap_generator_asarray__=True,
+            __enable_caching__=enable_virtualarray_caching,
         )
     # Unknown-length information implies that we didn't load shape-buffers (offsets, etc)
     # for the parent of this node. Thus, this node and its children *must* only
@@ -229,12 +261,12 @@ def _from_buffer(
     elif count is unknown_length:
         # We may actually have a known buffer here, but as we do not know the length,
         # we cannot safely trim it. Thus, introduce a placeholder anyway
-        return PlaceholderArray(nplike, (unknown_length,), dtype, field_path)
+        return PlaceholderArray(nplike, (unknown_length,), dtype, buffer_key)
     # Known-length information implies that we should have known-length buffers here
     # We could choose to make this an error, and have the caller re-implement some
     # of #ak.from_buffers, or we can just introduce the known lengths where possible
     elif isinstance(buffer, PlaceholderArray) and buffer.size is unknown_length:
-        return PlaceholderArray(nplike, (count,), dtype, field_path)
+        return PlaceholderArray(nplike, (count,), dtype, buffer_key)
     elif isinstance(buffer, PlaceholderArray) or nplike.is_own_array(buffer):
         # Require 1D buffers
         copy = None if isinstance(nplike, Jax) else False  # Jax can not avoid this
@@ -262,17 +294,17 @@ def _reconstitute(
     backend,
     byteorder,
     simplify,
-    field_path,
     shape_generator,
+    enable_virtualarray_caching,
 ):
     if isinstance(form, ak.forms.EmptyForm):
-        if length != 0:
+        if length is not unknown_length and length != 0:
             raise ValueError(f"EmptyForm node, but the expected length is {length}")
         return ak.contents.EmptyArray(backend=backend)
 
     elif isinstance(form, ak.forms.NumpyForm):
         dtype = ak.types.numpytype.primitive_to_dtype(form.primitive)
-        raw_array = container[getkey(form, "data")]
+        raw_array = container[(key := getkey(form, "data"))]
 
         def _adjust_length(length):
             return length * math.prod(form.inner_shape)
@@ -289,8 +321,11 @@ def _reconstitute(
             dtype=dtype,
             count=real_length,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key,
             shape_generator=_shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "data"
+            ),
         )
         if form.inner_shape != ():
             data = backend.nplike.reshape(data, (length, *form.inner_shape))
@@ -308,8 +343,8 @@ def _reconstitute(
             backend,
             byteorder,
             simplify,
-            field_path,
             shape_generator,
+            enable_virtualarray_caching,
         )
         if simplify:
             make = ak.contents.UnmaskedArray.simplified
@@ -318,7 +353,7 @@ def _reconstitute(
         return make(content, parameters=form._parameters)
 
     elif isinstance(form, ak.forms.BitMaskedForm):
-        raw_array = container[getkey(form, "mask")]
+        raw_array = container[(key := getkey(form, "mask"))]
 
         def _adjust_length(length):
             return math.ceil(length / 8.0)
@@ -338,8 +373,11 @@ def _reconstitute(
             dtype=index_to_dtype[form.mask],
             count=next_length,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key,
             shape_generator=_shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "mask"
+            ),
         )
         content = _reconstitute(
             form.content,
@@ -349,8 +387,8 @@ def _reconstitute(
             backend,
             byteorder,
             simplify,
-            field_path,
             shape_generator,
+            enable_virtualarray_caching,
         )
         if simplify:
             make = ak.contents.BitMaskedArray.simplified
@@ -369,15 +407,18 @@ def _reconstitute(
         )
 
     elif isinstance(form, ak.forms.ByteMaskedForm):
-        raw_array = container[getkey(form, "mask")]
+        raw_array = container[(key := getkey(form, "mask"))]
         mask = _from_buffer(
             backend.nplike,
             raw_array,
             dtype=index_to_dtype[form.mask],
             count=length,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key,
             shape_generator=shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "mask"
+            ),
         )
         content = _reconstitute(
             form.content,
@@ -387,8 +428,8 @@ def _reconstitute(
             backend,
             byteorder,
             simplify,
-            field_path,
             shape_generator,
+            enable_virtualarray_caching,
         )
         if simplify:
             make = ak.contents.ByteMaskedArray.simplified
@@ -402,15 +443,18 @@ def _reconstitute(
         )
 
     elif isinstance(form, ak.forms.IndexedOptionForm):
-        raw_array = container[getkey(form, "index")]
+        raw_array = container[(key := getkey(form, "index"))]
         index = _from_buffer(
             backend.nplike,
             raw_array,
             dtype=index_to_dtype[form.index],
             count=length,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key,
             shape_generator=shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "index"
+            ),
         )
 
         def _adjust_length(index):
@@ -431,8 +475,8 @@ def _reconstitute(
             backend,
             byteorder,
             simplify,
-            field_path,
             _shape_generator,
+            enable_virtualarray_caching,
         )
         if simplify:
             make = ak.contents.IndexedOptionArray.simplified
@@ -445,15 +489,18 @@ def _reconstitute(
         )
 
     elif isinstance(form, ak.forms.IndexedForm):
-        raw_array = container[getkey(form, "index")]
+        raw_array = container[(key := getkey(form, "index"))]
         index = _from_buffer(
             backend.nplike,
             raw_array,
             dtype=index_to_dtype[form.index],
             count=length,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key,
             shape_generator=shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "index"
+            ),
         )
 
         def _adjust_length(index):
@@ -478,8 +525,8 @@ def _reconstitute(
             backend,
             byteorder,
             simplify,
-            field_path,
             _shape_generator,
+            enable_virtualarray_caching,
         )
         if simplify:
             make = ak.contents.IndexedArray.simplified
@@ -492,16 +539,19 @@ def _reconstitute(
         )
 
     elif isinstance(form, ak.forms.ListForm):
-        raw_array1 = container[getkey(form, "starts")]
-        raw_array2 = container[getkey(form, "stops")]
+        raw_array1 = container[(key1 := getkey(form, "starts"))]
+        raw_array2 = container[(key2 := getkey(form, "stops"))]
         starts = _from_buffer(
             backend.nplike,
             raw_array1,
             dtype=index_to_dtype[form.starts],
             count=length,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key1,
             shape_generator=shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "starts"
+            ),
         )
         stops = _from_buffer(
             backend.nplike,
@@ -509,8 +559,11 @@ def _reconstitute(
             dtype=index_to_dtype[form.stops],
             count=length,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key2,
             shape_generator=shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "stops"
+            ),
         )
 
         def _adjust_length(starts, stops):
@@ -534,8 +587,8 @@ def _reconstitute(
             backend,
             byteorder,
             simplify,
-            field_path,
             _shape_generator,
+            enable_virtualarray_caching,
         )
         return ak.contents.ListArray(
             ak.index.Index(starts),
@@ -545,7 +598,7 @@ def _reconstitute(
         )
 
     elif isinstance(form, ak.forms.ListOffsetForm):
-        raw_array = container[getkey(form, "offsets")]
+        raw_array = container[(key := getkey(form, "offsets"))]
 
         def _shape_generator():
             (first,) = shape_generator()
@@ -557,8 +610,11 @@ def _reconstitute(
             dtype=index_to_dtype[form.offsets],
             count=length + 1,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key,
             shape_generator=_shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "offsets"
+            ),
         )
 
         # next length
@@ -581,8 +637,8 @@ def _reconstitute(
             backend,
             byteorder,
             simplify,
-            field_path,
             _shape_generator,
+            enable_virtualarray_caching,
         )
         return ak.contents.ListOffsetArray(
             ak.index.Index(offsets),
@@ -609,8 +665,8 @@ def _reconstitute(
             backend,
             byteorder,
             simplify,
-            field_path,
             _shape_generator,
+            enable_virtualarray_caching,
         )
         return ak.contents.RegularArray(
             content,
@@ -629,8 +685,8 @@ def _reconstitute(
                 backend,
                 byteorder,
                 simplify,
-                (*field_path, field),
                 shape_generator,
+                enable_virtualarray_caching,
             )
             for content, field in zip(form.contents, form.fields)
         ]
@@ -643,16 +699,19 @@ def _reconstitute(
         )
 
     elif isinstance(form, ak.forms.UnionForm):
-        raw_array1 = container[getkey(form, "tags")]
-        raw_array2 = container[getkey(form, "index")]
+        raw_array1 = container[(key1 := getkey(form, "tags"))]
+        raw_array2 = container[(key2 := getkey(form, "index"))]
         tags = _from_buffer(
             backend.nplike,
             raw_array1,
             dtype=index_to_dtype[form.tags],
             count=length,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key1,
             shape_generator=shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "tags"
+            ),
         )
         index = _from_buffer(
             backend.nplike,
@@ -660,8 +719,11 @@ def _reconstitute(
             dtype=index_to_dtype[form.index],
             count=length,
             byteorder=byteorder,
-            field_path=field_path,
+            buffer_key=key2,
             shape_generator=shape_generator,
+            enable_virtualarray_caching=enable_virtualarray_caching(
+                form.form_key, "index"
+            ),
         )
 
         def _adjust_length(index, tags, tag):
@@ -697,8 +759,8 @@ def _reconstitute(
                 backend,
                 byteorder,
                 simplify,
-                field_path,
                 _shape_generators[i],
+                enable_virtualarray_caching,
             )
             for i, content in enumerate(form.contents)
         ]
